from django.template.loader import render_to_string
import simplejson as json
import datetime

def get_fields(model):
    if hasattr(model, "knockout_fields"):
        fields = model.knockout_fields()
    else:
        try:
            fields = model.to_dict().keys()
        except Exception, e:
            fields = model._meta.fields

    return fields

def koModel(model, field_names=None, data=None):

    if type(model) == str:
        modelName = model
    else:
        modelName = model.__name__

    if field_names:
        fields = field_names
    else:
        fields = get_fields(model)

    if hasattr(model, "comparator"):
        comparator = str(model.comparator())
    else:
        comparator = 'id' 

    modelViewString = render_to_string("knockout_modeler/model.html", {'modelName': modelName, 'fields': fields, 'data': data, 'comparator': comparator} )

    return modelViewString

def koData(queryset, field_names):

    modelName = queryset[0].__class__.__name__    
    modelNameData = []

    if field_names:
        fields = field_names
    else:
        if hasattr(model, "knockout_fields"):
            fields = model.knockout_fields()
        else:
            try:
                fields = model.to_dict().keys()
            except Exception, e:
                fields = model._meta.fields

    for obj in queryset:
        temp_dict = dict()
        for field in fields:
            temp_dict[field] = getattr(obj, field)
        modelNameData.append(temp_dict)

    dthandler = lambda obj: obj.isoformat() if isinstance(obj, datetime.datetime)  or isinstance(obj, datetime.date) else None
    return "var " + modelName + "Data = " + json.dumps(modelNameData, default=dthandler) + ';'

def ko(queryset, field_names):

    koDataString = koData(queryset, field_names)
    koModelString = koModel(queryset[0].__class__.__name__, field_names, data=True)

    koString = koDataString + '\n' + koModelString

    return koString



from django import template
import simplejson as json
import datetime

register = template.Library()

def get_json_value(value, arg):
    """
    Get a json-safe value from an item.
    """
    dthandler = lambda obj: obj.isoformat() if isinstance(obj, datetime.datetime)  or isinstance(obj, datetime.date) else None
    return json.dumps(getattr(value, arg), default=dthandler)

register.filter(get_json_value)


dfrom django import template
import simplejson as json
import datetime
from knockout_modeler.ko import ko, koData, koModel, get_fields

register = template.Library()

def knockout(values):
    """
    Knockoutify a QuerySet!
    """

    modelClass = values[0].__class__
    if hasattr(modelClass, "knockout_fields"):
        field_names = values[0].knockout_fields()
    else:
        try:
            field_names = values[0].to_dict().keys()
        except Exception, e:
            field_names = values[0]._meta.fields

    return ko(values, field_names)

def knockout_data(values):
    """

    """

    modelClass = values[0].__class__
    if hasattr(modelClass, "knockout_fields"):
        field_names = values[0].knockout_fields()
    else:
        try:
            field_names = values[0].to_dict().keys()
        except Exception, e:
            field_names = values[0]._meta.fields

    return koData(values, field_names)

def knockout_model(values):
    """

    """

    modelClass = values[0].__class__
    if hasattr(modelClass, "knockout_fields"):
        field_names = values[0].knockout_fields()
    else:
        try:
            field_names = values[0].to_dict().keys()
        except Exception, e:
            field_names = values[0]._meta.fields

    return koModel(modelClass, field_names)

register.filter(knockout)
register.filter(knockout_data)
register.filter(knockout_model)